/**
*	WARNING: Do Not Remove This Section
*
*       $LastChangedRevision: 365 $
*       $LastChangedDate: 2010-02-04 12:27:47 +0100 (Thu, 04 Feb 2010) $
*       $LastChangedBy: ivan.zapreev $
*
*	MRMC is a model checker for discrete-time and continuous-time Markov
*	reward models. It supports reward extensions of PCTL and CSL (PRCTL
*	and CSRL), and allows for the automated verification of properties
*	concerning long-run and instantaneous rewards as well as cumulative
*	rewards.
*
*	Copyright (C) The University of Twente, 2004-2008.
*	Copyright (C) RWTH Aachen, 2008-2009.
*       Author: David N. Jansen
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Main contact:
*		Lehrstuhl für Informatik 2, RWTH Aachen University
*		Ahornstrasse 55, 52074 Aachen, Germany
*		E-mail: info@mrmc-tool.org
*
*       Old contact:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: This is for testing bscc.c.
*/

#include "bscc.h"
#include "read_tra_file.h"

#include "runtime.h"

#include <stdio.h>
#include <string.h>

int main(int argc, char *argv[])
{
        sparse * sp;
        TBSCCs * B;
        int ** result;
        unsigned long number, i;
        const int * map;
        bitset * bs;

        if ( 3 != argc )
                err_msg_2(err_PARAM, "main(%d,%p)", argc, (void *) argv,
                                EXIT_FAILURE);

        /* read a state space from a file */
        sp = read_tra_file(argv[1]);
        if ( NULL == sp )
                err_msg_2(err_CALLBY, "main(%d,%p)", argc, (void *) argv,
                                EXIT_FAILURE);

        if ( 0 == strcmp(argv[2], "R") )
                set_method_bscc(REC);
        else if ( 0 == strcmp(argv[2], "N") )
                set_method_bscc(NON_REC);
        else
                err_msg_2(err_PARAM, "main(%d,%p)", argc, (void *) argv,
                                EXIT_FAILURE);
        B = allocateTBSCCs(sp);
        if ( NULL == B )
                err_msg_2(err_CALLBY, "main(%d,%p)", argc, (void *) argv,
                                EXIT_FAILURE);
        
        bs = get_new_bitset(mtx_rows(sp));
        if ( NULL == bs )
                err_msg_2(err_CALLBY, "main(%d,%p)", argc, (void *) argv,
                                EXIT_FAILURE);
        if ( err_state_iserror(fill_bitset_one(bs)) )
                err_msg_2(err_CALLBY, "main(%d,%p)", argc, (void *) argv,
                                (free_bitset(bs), EXIT_FAILURE));

        result = getNewBSCCs(B, bs);
        if ( NULL == result )
                err_msg_2(err_CALLBY, "main(%d,%p)", argc, (void *) argv,
                                (free_bitset(bs), EXIT_FAILURE));
        map = getStatesBSCCsMapping(B);
        if ( NULL == map )
                err_msg_2(err_CALLBY, "main(%d,%p)", argc, (void *) argv,
                                (free_bitset(bs), EXIT_FAILURE));

        number = (unsigned long int) (void *) result[0];
        printf("I found %lu BSCC(s), namely:\n", number);
        for ( i = 1 ; i <= number ; i++ ) {
                printf("BSCC %d has ", result[i][0]);
                if ( 1 == result[i][1] ) {
                        printf("1 state: %d\n", result[i][2] + 1);
                        if ( map[result[i][2]] != result[i][0] )
                                printf("The mapping is inconsistent!\n");
                } else {
                        int j, count = 0;

                        printf("%d states: ", result[i][1]);
                        for ( j = 0 ; j < mtx_rows(sp) ; j++ )
                                if ( map[j] == result[i][0] ) {
                                        if ( 0 < count ) {
                                                printf(", ");
                                                if ( count % 8 == 0 )
                                                        putchar('\n');
                                        }
                                        printf("%d", j + 1);
                                        count++;
                                }
                        if ( result[i][1] != count )
                                printf(", which are in reality %d states",
                                                count);
                        putchar('\n');
                }
        }

        freeBSCCs(result);
        freeTBSCC(B);
        free_sparse_ncolse(sp);
        
        return EXIT_SUCCESS;
}
