/**
*	WARNING: Do Not Remove This Section
*
*	$LastChangedRevision: 254 $
*	$LastChangedDate: 2010-01-07 23:01:04 +0100 (Thu, 07 Jan 2010) $
*	$LastChangedBy: davidjansen $
*	
*	MRMC is a model checker for discrete-time and continuous-time Markov
*	reward models. It supports reward extensions of PCTL and CSL (PRCTL 
*	and CSRL), and allows for the automated verification of properties
*	concerning long-run and instantaneous rewards as well as cumulative
*	rewards.
*	
* 	Copyright (C) The University of Twente, 2004-2008.
* 	Copyright (C) RWTH Aachen, 2008-2009.
*       Author: David N. Jansen
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Main contact:
*		Lehrstuhl für Informatik 2, RWTH Aachen University
*		Ahornstrasse 55, 52074 Aachen, Germany
*		E-mail: info@mrmc-tool.org
*	
*       Old contact:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*       Source description: This is for testing the partition.
*       Uses: partition.h
*/

# include "partition.h"
#include "label.h"

#include <stdio.h>
#include <stdlib.h>

int main(void) {
        /*@only@*/ /*@null@*/ labelling * l = NULL;
        /*@only@*/ /*@null@*/ partition * P = NULL;
        /*@only@*/ /*@null@*/ double * d = NULL;

        /* create, as auxiliary, a set of labels */
        l = get_new_label(2, 10); /* 10 states, 2 labels */
        if ( 0 == add_label(l, "a") || 0 == add_label(l, "e") )
                err_msg_0(err_CALLBY, "main()",
                                (free_labelling(l), EXIT_FAILURE));
        set_label_bit(l, "a", 1  -1);
        set_label_bit(l, "a", 2  -1);
        set_label_bit(l, "a", 3  -1);
        set_label_bit(l, "a", 4  -1);
        set_label_bit(l, "a", 5  -1);
        set_label_bit(l, "e", 1  -1);
        set_label_bit(l, "e", 3  -1);
        set_label_bit(l, "e", 5  -1);
        set_label_bit(l, "e", 7  -1);
        set_label_bit(l, "e", 9  -1);
        /* create a new partition and split according to the labelling */
        printf("New partition with four blocks:\n");
        if ( (P = init_partition(l)) == NULL
                        || err_state_iserror(part_number_blocks(P))
                        || err_state_iserror(print_partition(P)) )
                err_msg_0(err_CALLBY, "main()",
                                (free_labelling(l), EXIT_FAILURE));
        free_labelling(l);

        /* split the partition, according to a set of reals */
        printf("Splitting the partition with rewards:\n");
        if ( (d = calloc((size_t) 10, sizeof(double))) == NULL )
                err_msg_0(err_MEMORY,"main()",(free_partition(P),EXIT_FAILURE));

        d[2  -1] = 0.5;
        d[4  -1] = 0.5;
        d[3  -1] = 0.7;
        d[9  -1] = 0.7;
        if ( err_state_iserror(part_refine_rewards(P, d)) )
                err_msg_0(err_CALLBY, "main()",
                                (free(d), free_partition(P), EXIT_FAILURE));
        /* part_refine_rewards may introduce splitters, but part_number_blocks
           requires that there be none: */
        P->first_Sp = NULL; P->first_PredCl = NULL;
        if ( err_state_iserror(part_number_blocks(P))
                        || err_state_iserror(print_partition(P)) )
                err_msg_0(err_CALLBY, "main()",
                                (free(d), free_partition(P), EXIT_FAILURE));

        if ( err_state_iserror(free_partition(P)) )
                err_msg_0(err_CALLBY, "main()", (free(d), EXIT_FAILURE));
        P = NULL;

        printf("New partition with one block:\n");
        l = get_new_label(1, 10); /* 10 states, 1 label */
        if ( 0 == add_label(l, "a") || (P = init_partition(l)) == NULL )
                err_msg_0(err_CALLBY, "main()",
                                (free(d), free_labelling(l), EXIT_FAILURE));
        if ( err_state_iserror(part_number_blocks(P))
                        || err_state_iserror(print_partition(P)) )
                err_msg_0(err_CALLBY, "main()", (free(d), free_partition(P),
                                free_labelling(l), EXIT_FAILURE));
        d[1  -1] = 0.1;
        d[5  -1] = 0.5;
        d[6  -1] = 0.6;
        d[7  -1] = 0.7;
        d[8  -1] = 0.8;
        d[10  -1] = 1.0;
        printf("Splitting the partition into many small sets:\n");
        if ( err_state_iserror(part_refine_rewards(P, d)) )
                err_msg_0(err_CALLBY, "main()", (free(d), free_labelling(l),
                                free_partition(P), EXIT_FAILURE));
        /* part_refine_rewards may introduce splitters, but part_number_blocks
           requires that there be none: */
        P->first_Sp = NULL; P->first_PredCl = NULL;
        if ( err_state_iserror(part_number_blocks(P))
                        || err_state_iserror(print_partition(P)) )
                err_msg_0(err_CALLBY, "main()", (free(d), free_labelling(l),
                                free_partition(P), EXIT_FAILURE));
        free(d);
        if ( err_state_iserror(free_partition(P)) )
                err_msg_0(err_CALLBY, "main()", (free_labelling(l),
                                EXIT_FAILURE));
        free_labelling(l);
        return EXIT_SUCCESS;
}
