/**
*	WARNING: Do Not Remove This Section
*
*       $LastChangedRevision: 415 $
*       $LastChangedDate: 2010-12-18 17:21:05 +0100 (Sa, 18. Dez 2010) $
*       $LastChangedBy: davidjansen $
*
*	MRMC is a model checker for discrete-time and continuous-time Markov
*	reward models. It supports reward extensions of PCTL and CSL (PRCTL
*	and CSRL), and allows for the automated verification of properties
*	concerning long-run and instantaneous rewards as well as cumulative
*	rewards.
*
*	Copyright (C) The University of Twente, 2004-2008.
*	Copyright (C) RWTH Aachen, 2008-2009.
*       Authors: Joost-Pieter Katoen, Maneesh Khattri, Ivan Zapreev, Tim Kemna,
*               David N. Jansen
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Main contact:
*		Lehrstuhl für Informatik 2, RWTH Aachen University
*		Ahornstrasse 55, 52074 Aachen, Germany
*		E-mail: info@mrmc-tool.org
*
*       Old contact:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*       Source description: Tests the sparse matrix structure and some
*               operations on it
*/

# include "foxglynn.h"
#include "read_tra_file.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[])
{
	int size=0,i;
	double *vec, *res;
	sparse *sp;

	printf("Works for files with the following format: \n");
	printf("No.OfStates\n");
	printf("SrcState DestState Value\n");
	printf("...\n");
	if(argc==3)
	{
		printf("Loading ....\n");
                sp = read_tra_file(argv[1]);
                if ( NULL == sp )
                        err_msg_0(err_CALLBY, "main()", EXIT_FAILURE);
                size = mtx_rows(sp);
                if ( 0 >= size )
                        err_msg_0(err_INCONSISTENT, "main()",
                                        (free_sparse_ncolse(sp), EXIT_FAILURE));
		printf("Loaded matrix:\n");
                if ( err_state_iserror(print_mtx_sparse(sp)) )
                        err_msg_0(err_CALLBY, "main()",
                                        (free_sparse_ncolse(sp), EXIT_FAILURE));

		vec=(double *)malloc(size*sizeof(double));
                if ( NULL == vec )
                        err_msg_0(err_MEMORY, "main()",
                                        (free_sparse_ncolse(sp), EXIT_FAILURE));
		res=(double *)malloc(size*sizeof(double));
                if ( NULL == res )
                        err_msg_0(err_MEMORY, "main()", (free(vec),
                                        free_sparse_ncolse(sp), EXIT_FAILURE));
                i = 0;
                do
		{
			vec[i]=2.0;
			res[i]=0.0;
		}
                while ( ++i < size );

                if ( 0 == strcmp(argv[2], "N") )
		{
			printf("Multiply a matrix to a vector\n");
                        if ( err_state_iserror(multiply_mtx_MV(sp, vec, res)) )
                                err_msg_0(err_CALLBY, "main()",
                                                (free(res), free(vec),
                                                free_sparse_ncolse(sp),
                                                EXIT_FAILURE));
		}
                else if ( 0 == strcmp(argv[2], "T") )
			{
				printf("Multiply a vector to a matrix\n");
                                if ( err_state_iserror(multiply_mtx_TMV(sp, vec,
                                                        res)) )
                                        err_msg_0(err_CALLBY, "main()",
                                                        (free(res), free(vec),
                                                        free_sparse_ncolse(sp),
                                                        EXIT_FAILURE));
			}
                if ( err_state_iserror(print_vec_double(size, vec)) )
                        err_msg_0(err_CALLBY, "main()", (free(res), free(vec),
                                        free_sparse_ncolse(sp), EXIT_FAILURE));
		printf("\nResult: \n");
                if ( err_state_iserror(print_vec_double(size, res)) )
                        err_msg_0(err_CALLBY, "main()", (free(res), free(vec),
                                        free_sparse_ncolse(sp), EXIT_FAILURE));
		printf("\n");
                free(res);
                free(vec);
                free_sparse_ncolse(sp);
	}else{
                printf("FORMAT: ./test_sparse input_file.tra [N|T]\n");
	}
        return EXIT_SUCCESS;
}
