#
#	WARNING: Do Not Remove This Section
#
#       $LastChangedRevision: 365 $
#       $LastChangedDate: 2010-02-04 12:27:47 +0100 (Do, 04. Feb 2010) $
#       $LastChangedBy: ivan.zapreev $
#
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: The input is the list of files from which the gnuplot
#	data file will be constructed. The first file name is the x axes values file
#	(*.param file) other files shoud be statistics files (*.performance.statistics, *.memory.statistics)
#
#	NOTE: All of the files should have just one column of values and the
#	number of rows in it should be equal among all the files!
#	NOTE: There is a variable parameter out_file_name that sould be passed as
#	-v out_file_name="file name" to the script
#	NOTE: There is a -v normalization_factor="..." parameter that should be a number
#	NOTE: There is a -v normalization_factor="..." parameter that should be a number
#	it is used to normilize the resulting output of the script
#
#	--Example 01: For performance analysis MRMC produces elapsed-time
#	output time in milliseconds, sometimes we want to have results in
#	seconds, then we should use:
#		-v normalization_factor=1000,
#	For minutes:
#		-v normalization_factor=60000,
#	etc ...
#
#	--Example 02: For the pick memory usage 'ps' produces results in Kb
#	sometimes we want to have results in Mb, then we should use:
#		-v normalization_factor=1024.
#	For Gb:
#		-v normalization_factor=1048576,
#	etc ...
#
#	NOTE: This script heavily relies on the assumption that the first input
#       file is always the file with the x axis values

BEGIN{
	input_data_files = 0;
	first = 1;		#The first input file
}

{
        for ( i = 1 ; i <= NF ; i++ ) {
                if ( "" == $i )
                        continue;
                file_names[input_data_files] = $i;
		records = 0;
                print "    Reading data file: " $i;
                while ( ( getline line < $i ) > 0){
			if( first != 1){
                                #Note the input files are in some data units;
                                #for us it is
				#sometimes convenient to use other units
				data[input_data_files, records] = line / normalization_factor;
			}else{
				#This is the x axes values file
				data[input_data_files, records] = line;
			}
			records++;
		}
		input_data_files++;
                close($i)
		first = 0;		#The first input file was processed
	}
}

END {
        #Output the input data into the output file for gnuplot
	print "----------------------------------------------------";
	print "    Writing gnuplot-data file: " out_file_name;

	#Print the headings
	for ( y = 0; y < input_data_files; y++){
		printf "#%s", file_names[y] >> out_file_name;
		if ( y != ( input_data_files-1 ) ){
			printf "\t" >> out_file_name;
		}
	}
	print "" >> out_file_name;

        #Print the results row-column wise
	for ( x = 0; x < records; x ++){
		for ( y = 0; y < input_data_files; y++){
			printf "%g", data[y, x] >> out_file_name;
			#Add separators between the columns
			if ( y != ( input_data_files-1 ) ){
				printf "\t\t" >> out_file_name;
				#Make things look more or less in one column
				if ( data[y, x] < 1000000 ) {
					printf "\t" >> out_file_name;
				}
			}
		}
		print "" >> out_file_name;
	}
}
